"use strict";

const mdTransformer = require('gatsby-transformer-remark/gatsby-node');

const toml = require('toml');

const jsYaml = require('js-yaml');

const createMdxNode = require('./create-mdx-node');

const mappedTypeName = require('./mapped-type-name');

module.exports = async (props, options) => {
  const {
    node,
    getNode,
    createNodeId,
    createContentDigest,
    loadNodeContent,
    actions,
    reporter
  } = props; // options

  const {
    types = {},
    basePath = '/',
    debug
  } = options;
  const {
    createNode,
    createParentChildLink
  } = actions;
  const debugInfo = debug ? reporter.info : () => {
    return;
  }; // Only process files that match the types being configured in this plugin

  debugInfo(`[sourceInstanceName:${node.sourceInstanceName}, type: ${node.internal.type}, mediaType:${node.internal.mediaType}]`);

  if (node.internal.type === "Mdx") {
    try {
      return createMdxNode(props, options);
    } catch (err) {
      reporter.panicOnBuild(`Error processing file ${node.absolutePath}\n${err.message}`);
    }

    return;
  }

  if (!node.sourceInstanceName || node.internal.type !== "File") return;
  const notPluginType = Boolean(!types[node.sourceInstanceName]);
  debugInfo(`${notPluginType ? 'NOT Processing' : 'Processing'} ${node.internal.type} ${node.sourceInstanceName}`);
  if (notPluginType) return;

  const createJsonNode = async parse => {
    try {
      const content = await loadNodeContent(node);
      const parsedContent = parse(content);
      const typeName = mappedTypeName(types)(node.sourceInstanceName);
      const jsonNode = Object.assign({}, parsedContent, {
        id: createNodeId(`${node.id} >>> ${typeName}`),
        children: [],
        parent: node.id,
        internal: {
          contentDigest: createContentDigest(parsedContent),
          type: `${typeName}`
        }
      });
      createNode(jsonNode);
      createParentChildLink({
        parent: node,
        child: jsonNode
      });
      debugInfo(`Created Node ${typeName} item: ${jsonNode.id}`);
      return jsonNode;
    } catch (err) {
      reporter.panicOnBuild(`Error processing ${node.sourceInstanceName ? `${mappedTypeName(types)(node.sourceInstanceName)} from file ${node.absolutePath}` : `file ${node.absolutePath}`}:\n
        ${err.message}`);
      return {};
    }
  }; // Ensures we are processing only files from the plugin setup (collections)


  if (node.internal.mediaType === 'application/json') {
    return createJsonNode(JSON.parse);
  }

  if (node.internal.mediaType === 'text/yaml') {
    return createJsonNode(jsYaml.load);
  }

  if (node.extension === 'toml') {
    return createJsonNode(toml.parse);
  }

  if (node.internal.mediaType === `text/markdown` || node.internal.mediaType === `text/x-markdown`) {
    try {
      const markdownNode = await mdTransformer.onCreateNode(props, {});
      const typeObj = types[node.sourceInstanceName];
      const typeName = mappedTypeName(types)(node.sourceInstanceName);
      const slug = `${typeObj.contentPath || typeName.toLowerCase()}/${node.name}`;
      const newNode = {
        id: createNodeId(`${node.id} >>> ${typeName}`),
        children: [],
        parent: node.id,
        internal: {
          type: `${typeName}`
        },
        name: `${node.name}`,
        slug,
        rawBody: markdownNode.rawMarkdownBody,
        frontmatter: markdownNode.frontmatter,
        fileAbsolutePath: markdownNode.fileAbsolutePath
      };
      newNode.internal.contentDigest = createContentDigest(newNode);
      createNode(newNode);
      createParentChildLink({
        parent: markdownNode,
        child: newNode
      });
      debugInfo(`Created Node ${typeName} item: ${newNode.id}`);
      return newNode;
    } catch (err) {
      reporter.panicOnBuild(`Error processing ${node.sourceInstanceName ? `${mappedTypeName(types)(node.sourceInstanceName)} from file ${node.absolutePath}` : `file ${node.absolutePath}`}:\n
        ${err.message}`);
      return {};
    }
  }
};